@@ -52,7 +52,7 @@ class carve(PatternExtractor):
         elif self.args.format is formats.b64:
             from ..encoding.b64 import b64
             decoder = b64()
-        elif self.args.format is formats.b64u:
+        elif self.args.format is formats.b64url:
             from ..encoding.b64 import b64
             decoder = b64(urlsafe=True)
         elif self.args.format is formats.b32:
@@ -67,6 +67,9 @@ class carve(PatternExtractor):
         elif self.args.format is formats.vbe:
             from ..encoding.vbe import vbe
             decoder = vbe()
+        elif self.args.format in (formats.urlenc, formats.urlencstrict):
+            from ..encoding.url import url
+            decoder = url()
         else:
             decoder = NotImplemented
         self.decoder = decoder
