@@ -4,12 +4,11 @@ import sys
 import textwrap
 import codecs
 import string
-import itertools
 
-from . import arg, HexViewer, get_terminal_size
+from . import arg, HexViewer
 from ...lib.meta import GetMeta, CustomStringRepresentation, SizeInt
 from ...lib.types import INF
-from ...lib.tools import isbuffer
+from ...lib.tools import isbuffer, lookahead
 
 
 class peek(HexViewer):
@@ -42,8 +41,6 @@ class peek(HexViewer):
             meta=meta,
             width=width,
         )
-        self._sep = True
-        self._idx = None
 
     def process(self, data):
         for line in self._peeklines(data):
@@ -125,35 +122,40 @@ class peek(HexViewer):
 
     def _peeklines(self, data):
 
-        dump = None
-        termsize = get_terminal_size()
-        inner_width = width = self.args.width or termsize
-        working_codec = None
+        codec = None
+        lines = None
+        if data.temp:
+            last, index = data.temp
 
-        if self.args.brief:
-            inner_width -= SizeInt.width + 2
-            if self._idx is not None:
-                inner_width -= 6
+        if not self.args.brief:
+            padding = 0
+        else:
+            padding = SizeInt.width + 2
+            if index is not None:
+                padding += 6
+
+        metrics = self._get_metrics(len(data), self.args.lines, padding)
 
         if self.args.lines and data:
             if self.args.esc:
-                dump = self._trydecode(data, None, inner_width)
+                lines = self._trydecode(data, None, metrics.width)
             if self.args.decode:
                 for codec in ('UTF8', 'UTF-16LE', 'UTF-16', 'UTF-16BE'):
-                    dump = self._trydecode(data, codec, inner_width)
-                    if dump:
-                        working_codec = codec
+                    lines = self._trydecode(data, codec, metrics.width)
+                    if lines:
+                        codec = codec
                         break
-            if dump is None:
-                total = abs(self.args.lines * termsize // 3)
-                dump = self.hexdump(data, total=total, width=inner_width)
-                dump = list(itertools.islice(dump, 0, abs(self.args.lines)))
-                width = max(len(line) for line in dump)
+            if lines is None:
+                lines = list(self.hexdump(data, metrics))
 
         def separator(title=None):
-            if title is None or width <= len(title) + 8:
-                return width * '-'
-            return F'--{title}' + '-' * (width - len(title) - 2)
+            if title is None or metrics.width <= len(title) + 8:
+                return metrics.width * '-'
+            return F'--{title}' + '-' * (metrics.width - len(title) - 2)
+
+        if index is None:
+            yield separator()
+            return
 
         if not self.args.brief:
             meta = GetMeta(data)
@@ -163,27 +165,23 @@ class peek(HexViewer):
                 entropy_percent = meta['entropy'] * 100.0
                 meta['entropy'] = F'{entropy_percent:.2f}%'
                 meta['size'] = meta['size']
-            yield from self._peekmeta(width, separator(), **meta)
+            yield from self._peekmeta(metrics.width, separator(), **meta)
 
-        if dump:
+        if lines:
             if not self.args.brief:
-                yield separator(F'CODEC={working_codec}' if working_codec else None)
-                yield from dump
+                yield separator(F'CODEC={codec}' if codec else None)
+                yield from lines
             else:
-                brief = next(iter(dump))
+                brief = next(iter(lines))
                 brief = F'{SizeInt(len(data))!r}: {brief}'
                 if self._idx is not None:
                     brief = F'#{self._idx:03d}: {brief}'
                 yield brief
 
-        if self._sep and not self.args.brief:
+        if last:
             yield separator()
 
-    def filter(self, inputs):
-        from ...lib.tools import lookahead
-        for last, (index, item) in lookahead(enumerate(inputs)):
-            self._sep = last
-            if not last or index:
-                self._idx = index
+    def filter(self, chunks):
+        for last, (index, item) in lookahead(enumerate(chunks)):
+            item.temp = last, index
             yield item
-        self._idx = None
